//Illinois Open Source License
//
//University of Illinois
//Open Source License
//
//Copyright © 2014,    Board of Trustees of the University of Illinois.  All rights reserved.
//
//Developed by:
//
// Distributed Protocols Research Group in the Department of Computer Science
// The University of Illinois at Urbana-Champaign
// http://dprg.cs.uiuc.edu/

//Imranul Hoque, ihoque2@illinois.edu
//Indranil Gupta, indy@cs.uiuc.edu
//
//Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the “Software”), to deal with the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:
//
//    * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimers.
//    * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimers in the documentation and/or other materials provided with the distribution.
//    * Neither the names of The Monet Group or The University of Illinois at Urbana-Champaign, nor the names of its contributors may be used to endorse or promote products derived from this Software without specific prior written permission.
//
//THE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN
//AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS WITH THE SOFTWARE.


package dprg;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintStream;
import java.net.Socket;
import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.nio.IntBuffer;

public class DegreeServerThread implements Runnable{
	private Socket server;
	int machine_count;
	int vertex_count;
	ByteBuffer []byteBuffer = null;
	IntBuffer []intBuffer = null;
	
	
	BufferedReader in = null;
	PrintStream out = null;

	public DegreeServerThread(Socket server, int machine_count, int vertex_count, int[][] degree) {
		super();
		this.server = server;
		this.machine_count = machine_count;
		this.vertex_count = vertex_count;
		byteBuffer = new ByteBuffer[machine_count];
		intBuffer = new IntBuffer[machine_count];
		for(int i = 0 ; i < machine_count ; i++) {
			byteBuffer[i] = ByteBuffer.allocate(degree[i].length * Integer.SIZE / Byte.SIZE);
			byteBuffer[i].order(ByteOrder.LITTLE_ENDIAN);
			intBuffer[i] = byteBuffer[i].asIntBuffer();
			intBuffer[i].put(degree[i]);
		}
	}
	public void run() {
		try {
			in = new BufferedReader(new InputStreamReader(
			        server.getInputStream()));
			int machine_id = in.read();
			System.out.println("Received request from: " + machine_id);
			out = new PrintStream(server.getOutputStream());		
			out.write(byteBuffer[machine_id].array());
			System.out.println("Sent degree distribution to:" + machine_id + ", count = " + byteBuffer[machine_id].array().length/4);
			server.close();
			
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
	}

}
