//Illinois Open Source License
//
//University of Illinois
//Open Source License
//
//Copyright © 2014,    Board of Trustees of the University of Illinois.  All rights reserved.
//
//Developed by:
//
// Distributed Protocols Research Group in the Department of Computer Science
// The University of Illinois at Urbana-Champaign
// http://dprg.cs.uiuc.edu/

//Imranul Hoque, ihoque2@illinois.edu
//Indranil Gupta, indy@cs.uiuc.edu
//
//Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the “Software”), to deal with the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:
//
//    * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimers.
//    * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimers in the documentation and/or other materials provided with the distribution.
//    * Neither the names of The Monet Group or The University of Illinois at Urbana-Champaign, nor the names of its contributors may be used to endorse or promote products derived from this Software without specific prior written permission.
//
//THE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN
//AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS WITH THE SOFTWARE.


#ifndef LANGAR_H_
#define LANGAR_H_

#include "vertex.h"
#include "barrier.h"
#include "timer.h"
#include <iostream>
#include <algorithm>
#include <fstream>
#include <tr1/unordered_map>
#include <map>
#include <bitset>
#include <cfloat>
#include "boost/threadpool.hpp"
#include <algorithm>
#include <boost/thread/thread.hpp>
#include <boost/dynamic_bitset.hpp>
#include <boost/asio.hpp>
#include <boost/asio/read.hpp>
#include <boost/random.hpp>
#include <boost/random/normal_distribution.hpp>

using namespace std;
using boost::asio::ip::tcp;

typedef boost::shared_ptr<tcp::socket> socket_ptr;

class Index {
	int current_size;
	int batch_size;
public:
	Index() {
		batch_size = 10000;
		current_size = batch_size;
		data = (int *) malloc(current_size * sizeof(int));
		i = 0;
	}
	~Index() {
		if(data) {
			free(data);
			data = NULL;
		}
	}
	int i;
	int *data;
	void add(int id) {
		if(i >= current_size) {
			current_size += batch_size;
			int *new_data = (int *) realloc(data, sizeof(int) * current_size);
			assert(new_data != NULL);
			data = new_data;
		}
		data[i++] = id;
	}
	void clear() {
		i = 0;
	}
	int getSize() {
		return i;
	}
};

class Host {
	char address[128];
	int port;
	int machine_id;

public:
	Host(const char* address, int port, int machine_id) {
		strcpy(this->address, address);
		this->port = port;
		this->machine_id = machine_id;
	}
	Host() {
		this->port = 10000;
		machine_id = 0;
	}
	Host(const Host &host) {
		strcpy(this->address, host.address);
		this->port = host.port;
		this->machine_id = host.machine_id;
	}
	Host& operator=(const Host &host) {
		strcpy(this->address, host.address);
		this->port = host.port;
		this->machine_id = host.machine_id;
		return *this;
	}
	int getMachine_id() {
		return this->machine_id;
	}
	int getPort() {
		return this->port;
	}
	char *getAddress() {
		return this->address;
	}
	void set(const char *address, int port, int machine_id) {
		strcpy(this->address, address);
		this->port = port;
		this->machine_id = machine_id;
	}
	Index index;
};

class Langar {
	int vertex_count;
	int machine_id;
	int machine_count;
	int thread_count;
	char filename[256];
	char zkserver[256];
	int zkport;
	bool optimized;
	bool synthetic;
	Host *host;
	Host *out_host;
	Timer timer;
	Vertex **v;
	int *partition_vcount;
	double *read_buffer;
	//double *write_buffer;
	int *active_count;
	bool *updated;
	NewBarrier *barrier;
	//multimap<int, int> in_list;
	//tr1::unordered_map<int, vector<int> > in_list;
	bool volatile running;
	void load_vertex();
	void load_lognormal();
	void load(int, vector<long long> &, bool *);
	void server();
	void optimized_server();
	void session(socket_ptr);
	void optimized_session(socket_ptr);
	void compute();
	void optimized_compute();
	void optimized_client(int);
	void client(int);
	void scatter(int);
	void self_client();
	void shortest_path(int, int);
	void memcopy(int);
	int gid_to_lid(int);
	int gid_to_pid(int);
public:
	Langar(int, const char *, int, int, int, Host *, const char *, int, bool);
	~Langar();
	void pagerank(int, int);
	void optimized_worker(int, int);

};

#endif /* LANGAR_H_ */
