//Illinois Open Source License
//
//University of Illinois
//Open Source License
//
//Copyright © 2014,    Board of Trustees of the University of Illinois.  All rights reserved.
//
//Developed by:
//
// Distributed Protocols Research Group in the Department of Computer Science
// The University of Illinois at Urbana-Champaign
// http://dprg.cs.uiuc.edu/

//Imranul Hoque, ihoque2@illinois.edu
//Indranil Gupta, indy@cs.uiuc.edu
//
//Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the “Software”), to deal with the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:
//
//    * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimers.
//    * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimers in the documentation and/or other materials provided with the distribution.
//    * Neither the names of The Monet Group or The University of Illinois at Urbana-Champaign, nor the names of its contributors may be used to endorse or promote products derived from this Software without specific prior written permission.
//
//THE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN
//AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS WITH THE SOFTWARE.


#include "preprocess.h"
#include "vertex.h"

FileSplitter::FileSplitter(const char *filename, int num_split,
		int num_thread) {
	strcpy(this->filename, filename);
	this->num_split = num_split;
	this->num_thread = num_thread;
}
void FileSplitter::splitter(int id, int total_part, const char *ifilename,
		long long start_pos, long long end_pos) {
	ifstream ifile(ifilename, ios::binary);
	ofstream *ofile = new ofstream[total_part];

	for (int i = 0; i < total_part; i++) {
		char filename[256];
		sprintf(filename, "%s.%d.%d", ifilename, id, i);
		ofile[i].open(filename, ios::binary);
	}

	string line;

	if (start_pos != 0) {
		ifile.seekg(start_pos - 1, ios::beg);
		char c;
		ifile.read(&c, 1);
		if (c != '\n') {
			getline(ifile, line);
		}
	}

	while (true) {
		int id;
		ifile >> id;
		int destination = id % total_part;
		getline(ifile, line);
		ofile[destination] << id << line << endl;
		if (ifile.tellg() >= end_pos) {
			break;
		}
	}

	for (int i = 0; i < total_part; i++) {
		ofile[i].close();
	}
	ifile.close();

	delete[] ofile;
}

void FileSplitter::split() {

	ifstream file(filename, ios::binary | ios::ate);
	assert(file.is_open());
	streampos pos = file.tellg();
	file.close();

	boost::thread_group tgroup;
	for (int i = 0; i < num_thread; i++) {
		tgroup.create_thread(
				boost::bind(&FileSplitter::splitter, this, i, num_split,
						filename, (pos / num_thread) * i,
						(pos / num_thread) * (i + 1)));
	}
	tgroup.join_all();
	this->merge();
}

void FileSplitter::merger(int part) {
	char sfilename[128];
	char ofilename[128];
	sprintf(ofilename, "%s.%d", filename, part);
	char command[1024];
	sprintf(command, "rm %s", ofilename);
	cout << "Executing: " << command << endl;
	system(command);

	for (int i = 0; i < num_thread; i++) {
		sprintf(sfilename, "%s.%d.%d", filename, i, part);
		sprintf(command, "cat %s >> %s", sfilename, ofilename);
		cout << "Executing: " << command << endl;
		system(command);
		sprintf(command, "rm %s", sfilename);
		cout << "Executing: " << command << endl;
		system(command);
	}
}
void FileSplitter::merge() {
	boost::thread_group tgroup;
	for (int i = 0; i < num_split; i++) {
		tgroup.create_thread(boost::bind(&FileSplitter::merger, this, i));
	}
	tgroup.join_all();
}
FileSanitizer::FileSanitizer(const char *filename) {
	strcpy(this->filename, filename);
}

void FileSanitizer::sanitize() {
	ifstream fin(filename);
	assert(fin.is_open());

	char ofilename[128];
	sprintf(ofilename, "%s.out", filename);
	ofstream fout(ofilename);

	int batch_size = 10000;
	int current_size = batch_size;
	int vertex_count = 0;
	Vertex *v = (Vertex *) malloc(batch_size * sizeof(Vertex));

	for (int i = 0;; i++) {
		int id, fcount, fid;
		fin >> id;
		if (fin.eof()) {
			break;
		}
		if (vertex_count >= current_size) {
			current_size += batch_size;
			Vertex *new_v = (Vertex *) realloc(v,
					sizeof(Vertex) * current_size);
			assert(new_v != NULL);
			v = new_v;
		}
		v[i].id = id;
		index_map[id] = i;
		fin >> fcount;
		v[i].fcount = fcount;

		v[i].f_list = (int *) malloc(fcount * sizeof(int));
		for (int j = 0; j < fcount; j++) {
			fin >> fid;
			v[i].f_list[j] = fid;
		}

		vertex_count++;
	}

	for (int i = 0; i < vertex_count; i++) {
		fout << i << " " << v[i].fcount;
		for (int j = 0; j < v[i].fcount; j++) {
			int friend_id = index_map[v[i].f_list[j]];
			if (friend_id == 0 && v[i].f_list[j] != v[0].id) {
				if (vertex_count >= current_size) {
					current_size += batch_size;
					Vertex *new_v = (Vertex *) realloc(v,
							sizeof(Vertex) * current_size);
					assert(new_v != NULL);
					v = new_v;
				}
				v[vertex_count].id = v[i].f_list[j];
				index_map[v[i].f_list[j]] = vertex_count;
				v[vertex_count].fcount = 0;
				friend_id = vertex_count;
				vertex_count++;

			}
			fout << " " << friend_id;
		}
		fout << endl;
	}
	fin.close();
	fout.close();
}
